SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

ALTER      Proc up_WordIndex
as
--SET NOCOUNT ON
------------------------------------
/*
CreateDate: 		2006-03-15
CreateBy:		DEXLEAP.COM
CreatePurpose:

For each entry text, we loop from the first nvarchar to the last.
this is should be run as a batch job.

Memo: The word we index is at most 20 nvarchar long

1. Any CJK (ChineseJapaneseKorean) nvarchar is a stem for itself 
2. Any Latin word will do string-add from the previous Latin nvarchar, 
   this will be broken by next non Latin nvarchar
3. Any number will do string-add for the previous number nvarchar,
   this will be broken by next non number nvarchar
4. Any nvarchar not belong to the above 1,2,3 will be ignored.

5. The current indexing entry can be stopped because UI resets the entry index status
 
6. The current indexing process can be stopped because UI sets the stop sign for it.

7. This process has to be executed by one single process. It shouldn't be executed concurrently.

8. At any given time, there is only one indexing entry.

9. Everytime this process started, It will turn all the "indexing" status into "ready to index" status.
===================
RevisDate:
ReviseBy:
ReviseReason:
*/
------------------------------------
------------------------------------
--Customize Declaration
------------------------------------
declare
	@sWordToBeIndex nvarchar(20),
	@sCharToBeWord nchar(1), 
	@unicodeChar int,
	@ixLoopEntryText int,
	@ixLoopEntry int,
	@ixArchiveEntry int,
	@ixWordStartPos int,

	@ixArchiveEntryPeek int,
	@nIndexStatusPeek int,
	--control the indexing process continue or not
	@ynDoIndex varchar(3),

	@lenStringEntryText int,
	@sStringChunk nvarchar(3000)


create table #tEntryLoop
(
	--this field is for long-run index process jumping out
	ixArchiveEntryPeek int
)

create table #tWordChunk
(
	--ixWordStartPos-sWord pair has to be unique in one entry
	[ixWordIndex] int NULL,
	[sWord] nvarchar(20) NOT NULL,
	[ixWordStartPos] int NOT NULL
)

--group by twordchunk to get a UNIQUE sWord (only for import index purpose)
create table #tWordIndex
(
	[sWord] nvarchar(20) NOT NULL

)
------------------------------------
--Customize Initialize
------------------------------------
--all the "indexing" entries should be turned into "ready to index".
--because these indexing entries must be a faulty legacy from the last time run (power failure or something)

Update tdatArchiveEntry
Set
	nIndexStatus = 1
Where
	nIndexStatus = 2

--only those "ready to index" entries are going to be indexed
--NOTE: this is the first snapshot, some entries might change status during the indexing
--so we need to check it again, just in case we need to skip some indexing entries.
Insert into #tEntryLoop
(
	ixArchiveEntryPeek 
)
select	
	ixArchiveEntry
From
	tdatArchiveEntry entry
where
	nIndexStatus = 1
--those "ready to index" entries may have some indexed part before,
--delete them to avoid duplication.
Delete 
	tdatWordIndex2Text
From
	tdatWordIndex2Text dat
inner join
	#tEntryLoop tmp
on
	dat.ixArchiveEntry = tmp.ixArchiveEntryPeek


--normal init
Select
	@sWordToBeIndex = N'',
	@ixLoopEntryText =0,
	@ixLoopEntry = 0,
	@lenStringEntryText = 0,
	@ixArchiveEntry = 0

------------------------------------
--Main Logic
------------------------------------
--loop text  chunk by chunk
while(1=1)
Begin
	select	
		@ixLoopEntry = min(ixArchiveEntryPeek)
	From
		#tEntryLoop 
	where
		ixArchiveEntryPeek > @ixLoopEntry

	--loop broken condition
	if @ixLoopEntry is null 
	Begin
		--Mark the entry which has just done the index
		--(status=3)
		Update tdatArchiveEntry
		Set
			nIndexStatus = 3
		Where
			ixArchiveEntry = @ixArchiveEntry
		Break
	End
	
	--get the text's string and the length of the string
	select
		
		@lenStringEntryText = Datalength(sEntryText)/2, --unicode 2 bytes
		@ixArchiveEntry = ixArchiveEntry
	From
		tdatArchiveEntry
	Where
		ixArchiveEntry = @ixLoopEntry
	
	--Mark a flag saying that this entry is indexing (status = 2)
	Update tdatArchiveEntry
	Set
		nIndexStatus = 2
	Where
		ixArchiveEntry = @ixArchiveEntry


	--reset tmp tables
	--delete from #tWordChunk	
	--delete from #tWordIndexChunk
	--reset the chunk text loop whenever we move to a new chunk
	select 
		@ixLoopEntryText = 1,
		@sCharToBeWord = N'',
		@sWordToBeIndex = N''
	--loop text char by char
	while (@ixLoopEntryText <= @lenStringEntryText)
	Begin

		--might be performance hit here.
		Select
			@sCharToBeWord = substring(sEntryText,@ixLoopEntryText,1)
		From
			tdatArchiveEntry
		where
			ixArchiveEntry = @ixArchiveEntry	

		Select
			@unicodeChar = UNICODE(@sCharToBeWord)
		
		
		if 	--Chinese UNICODE charts' range (from www.unicode.org)
			((@unicodeChar>=0x4E00) and (@unicodeChar<=0x9FBF))
				or
			((@unicodeChar>=0x3400) and (@unicodeChar<=0x4DBF))
				or
			((@unicodeChar>=0x20000) and (@unicodeChar<=0x2A6DF))
				or
			((@unicodeChar>=0xF900) and (@unicodeChar<=0xFAFF))
				or
			((@unicodeChar>=0x2F800) and (@unicodeChar<=0x2FA1F))
			--Japanese UNICODE charts' range (from www.unicode.org)
				or
			((@unicodeChar>=0x3040) and (@unicodeChar<=0x309F))
				or
			((@unicodeChar>=0x30A0) and (@unicodeChar<=0x30FF))
				or
			((@unicodeChar>=0x31F0) and (@unicodeChar<=0x31FF))
			--Korean UNICODE charts' range (from www.unicode.org)
				or
			((@unicodeChar>=0xAC00) and (@unicodeChar<=0xD7AF))
				or
			((@unicodeChar>=0x1100) and (@unicodeChar<=0x11FF))
				or
			((@unicodeChar>=0x3130) and (@unicodeChar<=0x318F))

		Begin
			--insert CJK (ChinesJapaneseKorean) nchar
			insert into #tWordChunk (sWord,ixWordStartPos)	
			Values(@sCharToBeWord,@ixLoopEntryText)
			--see if it's necessary to append the previous @sWordToBeIndex
			if (@sWordToBeIndex is not null)	
			Begin
			--insert the previous @sWordToBeIndex to the table
			insert into #tWordChunk (sWord,ixWordStartPos)	
			Values(@sWordToBeIndex,@ixLoopEntryText)
			End
			--reset both
			select 
				@sCharToBeWord = N'',
				@sWordToBeIndex = N''
			
		End
		else if --Numbers and Latins
			((@unicodeChar>=0x0030) and (@unicodeChar<=0x0039))
				or
			((@unicodeChar>=0x0041) and (@unicodeChar<=0x005a))
				or
			((@unicodeChar>=0x0061) and (@unicodeChar<=0x007a))
				or
			((@unicodeChar>=0x00c0) and (@unicodeChar<=0x00ff))
				or
			((@unicodeChar>=0x0100) and (@unicodeChar<=0x017f))
				or
			((@unicodeChar>=0x0180) and (@unicodeChar<=0x0241))
		Begin
			--String Add
			select 
				@sWordToBeIndex = @sWordToBeIndex + @sCharToBeWord
		End
		Else -- If it's neither CJK nor Latin , it must be some kind of breakers
		Begin
			--see if it's necessary to insert previous @sWordToBeIndex
			if (@sWordToBeIndex <> N'')	
			Begin
			--insert the previous @sWordToBeIndex to the table
			insert into #tWordChunk (sWord,ixWordStartPos)	
			Values(@sWordToBeIndex,@ixLoopEntryText-len(@sWordToBeIndex))
			End
			--reset both
			select 
				@sCharToBeWord = N'',
				@sWordToBeIndex = N''
		End

		select 
			@ixLoopEntryText = @ixLoopEntryText + 1
		--here we give it a chance to quit the whole process
		Select
			
			@ynDoIndex = isnull(ynDoIndex,'no')--just in case 
		From
			tdatWordIndexControl
		--quit the whole process because UI said stopping index.
		if @ynDoIndex <> 'yes'
		Begin
			goto procend
		End
		

		/*Herebelow is also very important for the process having a chance to jump out current entry*/	
		--Peek it outside to see if the current indexing entry should be skipped?
		Select
			@ixArchiveEntryPeek = @ixLoopEntry
		--check the peeked entry's current index status
		--If it is just deleted,make the status = 0 as well	
		Select 
			@nIndexStatusPeek = isnull(nIndexStatus,0)
		From
			tdatArchiveEntry
		Where
			ixArchiveEntry = @ixArchiveEntryPeek 

		--We might skip the current entry and jump out to the outer loop	
		--because other session might set this value to "don't index"	
		If @nIndexStatusPeek = 0
		Begin
			Delete From #tEntryLoop
			Where
				ixArchiveEntryPeek = @ixArchiveEntryPeek
			--jump to outter loop
			Break
		End
	End

	--blank word should be removed
	Delete 
	from 
		#tWordChunk 
	where
		sWord = N''



	--remove the noise
	Delete 	
	From 
		#tWordChunk 
	Where sWord in
		(
		Select
			sWordNoise
		From
			trefWordNoise 
		)
	
	--get unique sWord
	Insert into #tWordIndex
	(
		sWord
	)
	Select
		sWord
	From
		#tWordChunk			
	Group by
		sWord
		
	--if there are no existing ixWordIndex, We create them
	Insert into tdatWordIndex
		(
		 sWord
		)
	Select
		tmp.sWord	
	From
		#tWordIndex tmp
	left join
		tdatWordIndex dat
	on
		tmp.sWord = dat.sWord
	where
		dat.sWord is null	

	--get [ixWordIndex] for the stringchunk
	update tmp
	Set 
		tmp.ixWordIndex = dat.ixWordIndex
	From
		#tWordChunk tmp
	inner join
		tdatWordIndex dat 
	on
		tmp.sWord = dat.sWord
	--index2Text
	insert into tdatWordIndex2Text
		(
		 ixArchiveEntry,
		 ixWordIndex,
		 ixWordStartPos	
		)
	Select
		@ixArchiveEntry,
		ixWordIndex,
		ixWordStartPos
	From
		#tWordChunk		

	--Mark a flag saying that the entry has just done the index (status=3)
	Update tdatArchiveEntry
	Set
		nIndexStatus = 3
	Where
		ixArchiveEntry = @ixArchiveEntry
	--clean up word chunk temp tables
	delete from #tWordChunk
	delete from #tWordIndex

		
End

------------------------------------
--OutPut
------------------------------------
procend:



















GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

